@@ -62,12 +62,13 @@ def f_grad(w, X, Y, drifts, size_u, size_v):
     """Returns function AND gradient of the rank-one model"""
     u, v, bias = w[:size_u], w[size_u:size_u + size_v], w[size_u + size_v:]
     assert len(bias) == drifts.shape[1]
-    alpha = 1e-3
+    alpha = 1e6
     res = Y.ravel() - X.dot(np.outer(u, v).ravel('F')).ravel() - drifts.dot(bias)
     cost = .5 * linalg.norm(res) ** 2
-    cost -= alpha * .5 * (linalg.norm(u) ** 2)
+#    cost -= alpha * .5 * (linalg.norm(u[0]) ** 2)
     grad = np.empty((size_u + size_v + drifts.shape[1]))
-    grad[:size_u] = IaXb(X, v, res).ravel() + alpha * u
+    grad[:size_u] = IaXb(X, v, res).ravel()
+#    grad[0] +=  alpha * u[0]
     grad[size_u:size_u + size_v] = aIXb(X, u, res).ravel()
     grad[size_u + size_v:] = drifts.T.dot(res)
     return cost, -grad
@@ -149,7 +150,7 @@ def f_grad_separate(w, X, Y, size_u, size_v):
 
 def rank_one(X, y_i, n_basis,  w_i=None, drifts=None, callback=None, 
     maxiter=500, method='L-BFGS-B', rtol=1e-6,  verbose=0, mode='r1glm',
-    hrfs=None):
+    hrfs=None, basis=None):
     """
     Run a rank-one model with a given design matrix
 
@@ -192,6 +193,12 @@ def rank_one(X, y_i, n_basis,  w_i=None, drifts=None, callback=None,
             w_i = np.random.randn(n_basis + 2 * size_v, n_task)
         else:
             raise NotImplementedError
+    w_i = np.array(w_i)
+    if mode == 'r1glm':
+        assert w_i.shape[0] == n_basis + size_v + drifts.shape[1]
+    elif mode == 'r1glms':
+        assert w_i.shape[0] == n_basis + 2 * size_v + drifts.shape[1]
+    assert w_i.shape[1] == n_task
 
     if mode == 'r1glms':
         E = np.kron(np.ones((size_v, 1)), np.eye(n_basis))
@@ -214,7 +221,12 @@ def rank_one(X, y_i, n_basis,  w_i=None, drifts=None, callback=None,
         else:
             raise NotImplementedError
 
-    bounds = [(-1, 1)] * n_basis + [(None, None)] * (w_i.shape[0] - n_basis)
+    if basis is None:
+        bounds = [(1, 1)] + [(None, None)] * (w_i.shape[0] - 1)
+    elif basis in ('2hrf', '3hrf'):
+        # constrain the derivatives to not go too far
+        bounds = [(1, 1)] + [(-1., 1.)] * (n_basis - 1)  + \
+            [(None, None)]* (w_i.shape[0] - n_basis)
 
     if method == 'L-BFGS-B':
         solver = optimize.fmin_l_bfgs_b
@@ -233,7 +245,9 @@ def rank_one(X, y_i, n_basis,  w_i=None, drifts=None, callback=None,
 
         out = solver(
             ofunc, w_i[:, i], args=args, bounds=bounds,
-            maxiter=maxiter, callback=callback, factr=rtol, **kwargs)
+            maxiter=maxiter, callback=callback, pgtol=rtol, 
+            maxfun=30000,
+            **kwargs)
 
         if verbose > 1 and not out[-1]['warnflag'] != 0:
             warnings.warn(out[-1]['task'])
@@ -397,7 +411,7 @@ def glm(conditions, onsets, TR, Y, drifts=None, basis='3hrf', mode='r1glm',
         out = Parallel(n_jobs=n_jobs)(
             delayed(rank_one)(
                 X_design, y_i, size_u, w_i, drifts=drifts, callback=callback, maxiter=maxiter,
-                method=method, rtol=rtol, verbose=verbose, mode=mode, hrfs=hrfs)
+                method=method, rtol=rtol, verbose=verbose, mode=mode, hrfs=hrfs, basis=basis)
             for y_i, w_i in zip(Y_split, W_init_split))
 
         counter = 0
