@@ -9,7 +9,7 @@ from joblib import Parallel, delayed, cpu_count
 from . import hrf
 
 def create_design_matrix(conditions, onsets, TR, n_scans, basis='3hrf',
-                         oversample=5, hrf_length=20):
+                         oversample=10, hrf_length=32):
     """
     Parameters
     ----------
@@ -41,11 +41,8 @@ def create_design_matrix(conditions, onsets, TR, n_scans, basis='3hrf',
     design_matrix_cols = []
     B = []
     for b in basis:
-        tmp_basis = b(np.arange(0, hrf_length, resolution))
-        # import pylab as pl
-        # pl.plot(np.arange(0, hrf_length, resolution), tmp_basis)
-        # pl.show()
-        # import ipdb; ipdb.set_trace()
+        # needs to be a multiple of oversample
+        tmp_basis = b(np.linspace(0, hrf_length, (hrf_length // TR) * oversample))
         B.append(tmp_basis)
     for c in unique_conditions:
         tmp = np.zeros(int(n_scans * TR/resolution))
@@ -55,7 +52,7 @@ def create_design_matrix(conditions, onsets, TR, n_scans, basis='3hrf',
         for b in B:
             col = np.convolve(b, tmp, mode='full')[:tmp.size]
             col = col.reshape((-1, oversample), order='C')
-            col = col.mean(1)
+            col = np.median(col, axis=1)
             design_matrix_cols.append(col)
 
     design_matrix = np.array(design_matrix_cols).T
